@@ -1,4 +1,4 @@
-/* $Id: uuid-generic.cpp 39123 2008-11-10 23:35:26Z bird $ */
+/* $Id: uuid-generic.cpp 46536 2009-04-27 09:25:37Z bird $ */
 /** @file
  * IPRT - UUID, Generic.
  */
@@ -37,6 +37,35 @@
 #include <iprt/err.h>
 
 
+/*******************************************************************************
+*   Global Variables                                                           *
+*******************************************************************************/
+/** Conversion table used by the conversion functions.
+ * 0xff if not a hex number, otherwise the value. */
+static const uint8_t g_au8Digits[256] =
+{
+    0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff,  0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff, /* 0..0f */
+    0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff,  0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff, /* 10..1f */
+    0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff,  0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff, /* 20..2f */
+    0x00,0x01,0x02,0x03, 0x04,0x05,0x06,0x07,  0x08,0x09,0xff,0xff, 0xff,0xff,0xff,0xff, /* 30..3f */
+    0xff,0x0a,0x0b,0x0c, 0x0d,0x0e,0x0f,0xff,  0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff, /* 40..4f */
+    0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff,  0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff, /* 50..5f */
+    0xff,0x0a,0x0b,0x0c, 0x0d,0x0e,0x0f,0xff,  0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff, /* 60..6f */
+    0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff,  0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff, /* 70..7f */
+    0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff,  0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff, /* 80..8f */
+    0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff,  0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff, /* 90..9f */
+    0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff,  0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff, /* a0..af */
+    0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff,  0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff, /* b0..bf */
+    0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff,  0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff, /* c0..cf */
+    0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff,  0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff, /* d0..df */
+    0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff,  0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff, /* e0..ef */
+    0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff,  0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff, /* f0..ff */
+};
+/** Conversion to string. */
+static const char g_achDigits[17] = "0123456789abcdef";
+
+
+
 /* WARNING: This implementation ASSUMES little endian. Does not work on big endian! */
 
 /* Remember, the time fields in the UUID must be little endian. */
@@ -123,7 +152,6 @@ RTDECL(int)  RTUuidCompareStr(PCRTUUID pUuid1, const char *pszString)
 
 RTDECL(int)  RTUuidToStr(PCRTUUID pUuid, char *pszString, size_t cchString)
 {
-    static const char s_achDigits[17] = "0123456789abcdef";
     uint32_t u32TimeLow;
     unsigned u;
 
@@ -147,44 +175,44 @@ RTDECL(int)  RTUuidToStr(PCRTUUID pUuid, char *pszString, size_t cchString)
      *             pUuid->Gen.au8Node[5]);
      */
     u32TimeLow = pUuid->Gen.u32TimeLow;
-    pszString[ 0] = s_achDigits[(u32TimeLow >> 28)/*& 0xf*/];
-    pszString[ 1] = s_achDigits[(u32TimeLow >> 24) & 0xf];
-    pszString[ 2] = s_achDigits[(u32TimeLow >> 20) & 0xf];
-    pszString[ 3] = s_achDigits[(u32TimeLow >> 16) & 0xf];
-    pszString[ 4] = s_achDigits[(u32TimeLow >> 12) & 0xf];
-    pszString[ 5] = s_achDigits[(u32TimeLow >>  8) & 0xf];
-    pszString[ 6] = s_achDigits[(u32TimeLow >>  4) & 0xf];
-    pszString[ 7] = s_achDigits[(u32TimeLow/*>>0*/)& 0xf];
+    pszString[ 0] = g_achDigits[(u32TimeLow >> 28)/*& 0xf*/];
+    pszString[ 1] = g_achDigits[(u32TimeLow >> 24) & 0xf];
+    pszString[ 2] = g_achDigits[(u32TimeLow >> 20) & 0xf];
+    pszString[ 3] = g_achDigits[(u32TimeLow >> 16) & 0xf];
+    pszString[ 4] = g_achDigits[(u32TimeLow >> 12) & 0xf];
+    pszString[ 5] = g_achDigits[(u32TimeLow >>  8) & 0xf];
+    pszString[ 6] = g_achDigits[(u32TimeLow >>  4) & 0xf];
+    pszString[ 7] = g_achDigits[(u32TimeLow/*>>0*/)& 0xf];
     pszString[ 8] = '-';
     u = pUuid->Gen.u16TimeMid;
-    pszString[ 9] = s_achDigits[(u >> 12)/*& 0xf*/];
-    pszString[10] = s_achDigits[(u >>  8) & 0xf];
-    pszString[11] = s_achDigits[(u >>  4) & 0xf];
-    pszString[12] = s_achDigits[(u/*>>0*/)& 0xf];
+    pszString[ 9] = g_achDigits[(u >> 12)/*& 0xf*/];
+    pszString[10] = g_achDigits[(u >>  8) & 0xf];
+    pszString[11] = g_achDigits[(u >>  4) & 0xf];
+    pszString[12] = g_achDigits[(u/*>>0*/)& 0xf];
     pszString[13] = '-';
     u = pUuid->Gen.u16TimeHiAndVersion;
-    pszString[14] = s_achDigits[(u >> 12)/*& 0xf*/];
-    pszString[15] = s_achDigits[(u >>  8) & 0xf];
-    pszString[16] = s_achDigits[(u >>  4) & 0xf];
-    pszString[17] = s_achDigits[(u/*>>0*/)& 0xf];
+    pszString[14] = g_achDigits[(u >> 12)/*& 0xf*/];
+    pszString[15] = g_achDigits[(u >>  8) & 0xf];
+    pszString[16] = g_achDigits[(u >>  4) & 0xf];
+    pszString[17] = g_achDigits[(u/*>>0*/)& 0xf];
     pszString[18] = '-';
-    pszString[19] = s_achDigits[pUuid->Gen.u8ClockSeqHiAndReserved >> 4];
-    pszString[20] = s_achDigits[pUuid->Gen.u8ClockSeqHiAndReserved & 0xf];
-    pszString[21] = s_achDigits[pUuid->Gen.u8ClockSeqLow >> 4];
-    pszString[22] = s_achDigits[pUuid->Gen.u8ClockSeqLow & 0xf];
+    pszString[19] = g_achDigits[pUuid->Gen.u8ClockSeqHiAndReserved >> 4];
+    pszString[20] = g_achDigits[pUuid->Gen.u8ClockSeqHiAndReserved & 0xf];
+    pszString[21] = g_achDigits[pUuid->Gen.u8ClockSeqLow >> 4];
+    pszString[22] = g_achDigits[pUuid->Gen.u8ClockSeqLow & 0xf];
     pszString[23] = '-';
-    pszString[24] = s_achDigits[pUuid->Gen.au8Node[0] >> 4];
-    pszString[25] = s_achDigits[pUuid->Gen.au8Node[0] & 0xf];
-    pszString[26] = s_achDigits[pUuid->Gen.au8Node[1] >> 4];
-    pszString[27] = s_achDigits[pUuid->Gen.au8Node[1] & 0xf];
-    pszString[28] = s_achDigits[pUuid->Gen.au8Node[2] >> 4];
-    pszString[29] = s_achDigits[pUuid->Gen.au8Node[2] & 0xf];
-    pszString[30] = s_achDigits[pUuid->Gen.au8Node[3] >> 4];
-    pszString[31] = s_achDigits[pUuid->Gen.au8Node[3] & 0xf];
-    pszString[32] = s_achDigits[pUuid->Gen.au8Node[4] >> 4];
-    pszString[33] = s_achDigits[pUuid->Gen.au8Node[4] & 0xf];
-    pszString[34] = s_achDigits[pUuid->Gen.au8Node[5] >> 4];
-    pszString[35] = s_achDigits[pUuid->Gen.au8Node[5] & 0xf];
+    pszString[24] = g_achDigits[pUuid->Gen.au8Node[0] >> 4];
+    pszString[25] = g_achDigits[pUuid->Gen.au8Node[0] & 0xf];
+    pszString[26] = g_achDigits[pUuid->Gen.au8Node[1] >> 4];
+    pszString[27] = g_achDigits[pUuid->Gen.au8Node[1] & 0xf];
+    pszString[28] = g_achDigits[pUuid->Gen.au8Node[2] >> 4];
+    pszString[29] = g_achDigits[pUuid->Gen.au8Node[2] & 0xf];
+    pszString[30] = g_achDigits[pUuid->Gen.au8Node[3] >> 4];
+    pszString[31] = g_achDigits[pUuid->Gen.au8Node[3] & 0xf];
+    pszString[32] = g_achDigits[pUuid->Gen.au8Node[4] >> 4];
+    pszString[33] = g_achDigits[pUuid->Gen.au8Node[4] & 0xf];
+    pszString[34] = g_achDigits[pUuid->Gen.au8Node[5] >> 4];
+    pszString[35] = g_achDigits[pUuid->Gen.au8Node[5] & 0xf];
     pszString[36] = '\0';
 
     return VINF_SUCCESS;
@@ -193,27 +221,6 @@ RTDECL(int)  RTUuidToStr(PCRTUUID pUuid, char *pszString, size_t cchString)
 
 RTDECL(int)  RTUuidFromStr(PRTUUID pUuid, const char *pszString)
 {
-    /* 0xff if not a hex number, otherwise the value. (Assumes UTF-8 encoded strings.) */
-    static const uint8_t s_aDigits[256] =
-    {
-        0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff,  0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff, /* 0..0f */
-        0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff,  0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff, /* 10..1f */
-        0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff,  0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff, /* 20..2f */
-        0x00,0x01,0x02,0x03, 0x04,0x05,0x06,0x07,  0x08,0x09,0xff,0xff, 0xff,0xff,0xff,0xff, /* 30..3f */
-        0xff,0x0a,0x0b,0x0c, 0x0d,0x0e,0x0f,0xff,  0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff, /* 40..4f */
-        0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff,  0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff, /* 50..5f */
-        0xff,0x0a,0x0b,0x0c, 0x0d,0x0e,0x0f,0xff,  0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff, /* 60..6f */
-        0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff,  0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff, /* 70..7f */
-        0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff,  0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff, /* 80..8f */
-        0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff,  0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff, /* 90..9f */
-        0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff,  0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff, /* a0..af */
-        0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff,  0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff, /* b0..bf */
-        0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff,  0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff, /* c0..cf */
-        0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff,  0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff, /* d0..df */
-        0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff,  0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff, /* e0..ef */
-        0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff,  0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff, /* f0..ff */
-    };
-
     /*
      * Validate parameters.
      */
@@ -221,7 +228,7 @@ RTDECL(int)  RTUuidFromStr(PRTUUID pUuid, const char *pszString)
     AssertPtrReturn(pszString, VERR_INVALID_PARAMETER);
 
 #define MY_CHECK(expr) do { if (RT_UNLIKELY(!(expr))) return VERR_INVALID_UUID_FORMAT; } while (0)
-#define MY_ISXDIGIT(ch) (s_aDigits[(ch) & 0xff] != 0xff)
+#define MY_ISXDIGIT(ch) (g_au8Digits[(ch) & 0xff] != 0xff)
     MY_CHECK(MY_ISXDIGIT(pszString[ 0]));
     MY_CHECK(MY_ISXDIGIT(pszString[ 1]));
     MY_CHECK(MY_ISXDIGIT(pszString[ 2]));
@@ -265,7 +272,7 @@ RTDECL(int)  RTUuidFromStr(PRTUUID pUuid, const char *pszString)
     /*
      * Inverse of RTUuidToStr (see above).
      */
-#define MY_TONUM(ch) (s_aDigits[(ch) & 0xff])
+#define MY_TONUM(ch) (g_au8Digits[(ch) & 0xff])
     pUuid->Gen.u32TimeLow = (uint32_t)MY_TONUM(pszString[ 0]) << 28
                           | (uint32_t)MY_TONUM(pszString[ 1]) << 24
                           | (uint32_t)MY_TONUM(pszString[ 2]) << 20
@@ -305,3 +312,168 @@ RTDECL(int)  RTUuidFromStr(PRTUUID pUuid, const char *pszString)
     return VINF_SUCCESS;
 }
 
+
+RTDECL(int)  RTUuidToUtf16(PCRTUUID pUuid, PRTUTF16 pwszString, size_t cwcString)
+{
+    static const char g_achDigits[17] = "0123456789abcdef";
+    uint32_t u32TimeLow;
+    unsigned u;
+
+    /* validate parameters */
+    AssertPtrReturn(pUuid, VERR_INVALID_PARAMETER);
+    AssertPtrReturn(pwszString, VERR_INVALID_PARAMETER);
+    AssertReturn(cwcString >= RTUUID_STR_LENGTH, VERR_INVALID_PARAMETER);
+
+    /*
+     * RTStrPrintf(,,"%08x-%04x-%04x-%02x%02x-%02x%02x%02x%02x%02x%02x",
+     *             pUuid->Gen.u32TimeLow,
+     *             pUuid->Gen.u16TimeMin,
+     *             pUuid->Gen.u16TimeHiAndVersion,
+     *             pUuid->Gen.u16ClockSeq & 0xff,
+     *             pUuid->Gen.u16ClockSeq >> 8,
+     *             pUuid->Gen.au8Node[0],
+     *             pUuid->Gen.au8Node[1],
+     *             pUuid->Gen.au8Node[2],
+     *             pUuid->Gen.au8Node[3],
+     *             pUuid->Gen.au8Node[4],
+     *             pUuid->Gen.au8Node[5]);
+     */
+    u32TimeLow = pUuid->Gen.u32TimeLow;
+    pwszString[ 0] = g_achDigits[(u32TimeLow >> 28)/*& 0xf*/];
+    pwszString[ 1] = g_achDigits[(u32TimeLow >> 24) & 0xf];
+    pwszString[ 2] = g_achDigits[(u32TimeLow >> 20) & 0xf];
+    pwszString[ 3] = g_achDigits[(u32TimeLow >> 16) & 0xf];
+    pwszString[ 4] = g_achDigits[(u32TimeLow >> 12) & 0xf];
+    pwszString[ 5] = g_achDigits[(u32TimeLow >>  8) & 0xf];
+    pwszString[ 6] = g_achDigits[(u32TimeLow >>  4) & 0xf];
+    pwszString[ 7] = g_achDigits[(u32TimeLow/*>>0*/)& 0xf];
+    pwszString[ 8] = '-';
+    u = pUuid->Gen.u16TimeMid;
+    pwszString[ 9] = g_achDigits[(u >> 12)/*& 0xf*/];
+    pwszString[10] = g_achDigits[(u >>  8) & 0xf];
+    pwszString[11] = g_achDigits[(u >>  4) & 0xf];
+    pwszString[12] = g_achDigits[(u/*>>0*/)& 0xf];
+    pwszString[13] = '-';
+    u = pUuid->Gen.u16TimeHiAndVersion;
+    pwszString[14] = g_achDigits[(u >> 12)/*& 0xf*/];
+    pwszString[15] = g_achDigits[(u >>  8) & 0xf];
+    pwszString[16] = g_achDigits[(u >>  4) & 0xf];
+    pwszString[17] = g_achDigits[(u/*>>0*/)& 0xf];
+    pwszString[18] = '-';
+    pwszString[19] = g_achDigits[pUuid->Gen.u8ClockSeqHiAndReserved >> 4];
+    pwszString[20] = g_achDigits[pUuid->Gen.u8ClockSeqHiAndReserved & 0xf];
+    pwszString[21] = g_achDigits[pUuid->Gen.u8ClockSeqLow >> 4];
+    pwszString[22] = g_achDigits[pUuid->Gen.u8ClockSeqLow & 0xf];
+    pwszString[23] = '-';
+    pwszString[24] = g_achDigits[pUuid->Gen.au8Node[0] >> 4];
+    pwszString[25] = g_achDigits[pUuid->Gen.au8Node[0] & 0xf];
+    pwszString[26] = g_achDigits[pUuid->Gen.au8Node[1] >> 4];
+    pwszString[27] = g_achDigits[pUuid->Gen.au8Node[1] & 0xf];
+    pwszString[28] = g_achDigits[pUuid->Gen.au8Node[2] >> 4];
+    pwszString[29] = g_achDigits[pUuid->Gen.au8Node[2] & 0xf];
+    pwszString[30] = g_achDigits[pUuid->Gen.au8Node[3] >> 4];
+    pwszString[31] = g_achDigits[pUuid->Gen.au8Node[3] & 0xf];
+    pwszString[32] = g_achDigits[pUuid->Gen.au8Node[4] >> 4];
+    pwszString[33] = g_achDigits[pUuid->Gen.au8Node[4] & 0xf];
+    pwszString[34] = g_achDigits[pUuid->Gen.au8Node[5] >> 4];
+    pwszString[35] = g_achDigits[pUuid->Gen.au8Node[5] & 0xf];
+    pwszString[36] = '\0';
+
+    return VINF_SUCCESS;
+}
+
+
+RTDECL(int)  RTUuidFromUtf16(PRTUUID pUuid, PCRTUTF16 pwszString)
+{
+
+    /*
+     * Validate parameters.
+     */
+    AssertPtrReturn(pUuid, VERR_INVALID_PARAMETER);
+    AssertPtrReturn(pwszString, VERR_INVALID_PARAMETER);
+
+#define MY_CHECK(expr) do { if (RT_UNLIKELY(!(expr))) return VERR_INVALID_UUID_FORMAT; } while (0)
+#define MY_ISXDIGIT(ch) (!((ch) & 0xff00) && g_au8Digits[(ch) & 0xff] != 0xff)
+    MY_CHECK(MY_ISXDIGIT(pwszString[ 0]));
+    MY_CHECK(MY_ISXDIGIT(pwszString[ 1]));
+    MY_CHECK(MY_ISXDIGIT(pwszString[ 2]));
+    MY_CHECK(MY_ISXDIGIT(pwszString[ 3]));
+    MY_CHECK(MY_ISXDIGIT(pwszString[ 4]));
+    MY_CHECK(MY_ISXDIGIT(pwszString[ 5]));
+    MY_CHECK(MY_ISXDIGIT(pwszString[ 6]));
+    MY_CHECK(MY_ISXDIGIT(pwszString[ 7]));
+    MY_CHECK(pwszString[ 8] == '-');
+    MY_CHECK(MY_ISXDIGIT(pwszString[ 9]));
+    MY_CHECK(MY_ISXDIGIT(pwszString[10]));
+    MY_CHECK(MY_ISXDIGIT(pwszString[11]));
+    MY_CHECK(MY_ISXDIGIT(pwszString[12]));
+    MY_CHECK(pwszString[13] == '-');
+    MY_CHECK(MY_ISXDIGIT(pwszString[14]));
+    MY_CHECK(MY_ISXDIGIT(pwszString[15]));
+    MY_CHECK(MY_ISXDIGIT(pwszString[16]));
+    MY_CHECK(MY_ISXDIGIT(pwszString[17]));
+    MY_CHECK(pwszString[18] == '-');
+    MY_CHECK(MY_ISXDIGIT(pwszString[19]));
+    MY_CHECK(MY_ISXDIGIT(pwszString[20]));
+    MY_CHECK(MY_ISXDIGIT(pwszString[21]));
+    MY_CHECK(MY_ISXDIGIT(pwszString[22]));
+    MY_CHECK(pwszString[23] == '-');
+    MY_CHECK(MY_ISXDIGIT(pwszString[24]));
+    MY_CHECK(MY_ISXDIGIT(pwszString[25]));
+    MY_CHECK(MY_ISXDIGIT(pwszString[26]));
+    MY_CHECK(MY_ISXDIGIT(pwszString[27]));
+    MY_CHECK(MY_ISXDIGIT(pwszString[28]));
+    MY_CHECK(MY_ISXDIGIT(pwszString[29]));
+    MY_CHECK(MY_ISXDIGIT(pwszString[30]));
+    MY_CHECK(MY_ISXDIGIT(pwszString[31]));
+    MY_CHECK(MY_ISXDIGIT(pwszString[32]));
+    MY_CHECK(MY_ISXDIGIT(pwszString[33]));
+    MY_CHECK(MY_ISXDIGIT(pwszString[34]));
+    MY_CHECK(MY_ISXDIGIT(pwszString[35]));
+    MY_CHECK(!pwszString[36]);
+#undef MY_ISXDIGIT
+#undef MY_CHECK
+
+    /*
+     * Inverse of RTUuidToUtf8 (see above).
+     */
+#define MY_TONUM(ch) (g_au8Digits[(ch) & 0xff])
+    pUuid->Gen.u32TimeLow = (uint32_t)MY_TONUM(pwszString[ 0]) << 28
+                          | (uint32_t)MY_TONUM(pwszString[ 1]) << 24
+                          | (uint32_t)MY_TONUM(pwszString[ 2]) << 20
+                          | (uint32_t)MY_TONUM(pwszString[ 3]) << 16
+                          | (uint32_t)MY_TONUM(pwszString[ 4]) << 12
+                          | (uint32_t)MY_TONUM(pwszString[ 5]) <<  8
+                          | (uint32_t)MY_TONUM(pwszString[ 6]) <<  4
+                          | (uint32_t)MY_TONUM(pwszString[ 7]);
+    pUuid->Gen.u16TimeMid = (uint16_t)MY_TONUM(pwszString[ 9]) << 12
+                          | (uint16_t)MY_TONUM(pwszString[10]) << 8
+                          | (uint16_t)MY_TONUM(pwszString[11]) << 4
+                          | (uint16_t)MY_TONUM(pwszString[12]);
+    pUuid->Gen.u16TimeHiAndVersion =
+                            (uint16_t)MY_TONUM(pwszString[14]) << 12
+                          | (uint16_t)MY_TONUM(pwszString[15]) << 8
+                          | (uint16_t)MY_TONUM(pwszString[16]) << 4
+                          | (uint16_t)MY_TONUM(pwszString[17]);
+    pUuid->Gen.u8ClockSeqHiAndReserved =
+                            (uint16_t)MY_TONUM(pwszString[19]) << 4
+                          | (uint16_t)MY_TONUM(pwszString[20]);
+    pUuid->Gen.u8ClockSeqLow =
+                            (uint16_t)MY_TONUM(pwszString[21]) << 4
+                          | (uint16_t)MY_TONUM(pwszString[22]);
+    pUuid->Gen.au8Node[0] = (uint8_t)MY_TONUM(pwszString[24]) << 4
+                          | (uint8_t)MY_TONUM(pwszString[25]);
+    pUuid->Gen.au8Node[1] = (uint8_t)MY_TONUM(pwszString[26]) << 4
+                          | (uint8_t)MY_TONUM(pwszString[27]);
+    pUuid->Gen.au8Node[2] = (uint8_t)MY_TONUM(pwszString[28]) << 4
+                          | (uint8_t)MY_TONUM(pwszString[29]);
+    pUuid->Gen.au8Node[3] = (uint8_t)MY_TONUM(pwszString[30]) << 4
+                          | (uint8_t)MY_TONUM(pwszString[31]);
+    pUuid->Gen.au8Node[4] = (uint8_t)MY_TONUM(pwszString[32]) << 4
+                          | (uint8_t)MY_TONUM(pwszString[33]);
+    pUuid->Gen.au8Node[5] = (uint8_t)MY_TONUM(pwszString[34]) << 4
+                          | (uint8_t)MY_TONUM(pwszString[35]);
+#undef MY_TONUM
+    return VINF_SUCCESS;
+}
+
